

import os
import tempfile
from seeqret import seeqret_init


def test_init():
    with tempfile.TemporaryDirectory() as temp_dir:
        seeqret_init.secrets_init(temp_dir)
        # seeqret_init.secrets_init()
        assert os.path.exists(os.path.join(temp_dir, '.seeqret'))
        print(temp_dir)



import gnupg

PASSPHRASE = 'correct horse battery staple'


def generate_keys(gpg, user):
    # gpg = gnupg.GPG(gnupghome=f'{user}/.gnupg')
    key = gpg.gen_key(
        gpg.gen_key_input(
            key_type="RSA",
            key_length=4096,
            name_email=f'{user}@example.com',
            passphrase=PASSPHRASE
        ))

    with open(f'{user}/.gnupg/gpg-public.asc', 'w') as f:
        pubkey = gpg.export_keys(key.fingerprint)
        print(pubkey.replace('\r', ''), file=f)

    with open(f'{user}/.gnupg/gpg-private.asc', 'w') as f:
        privkey = gpg.export_keys(key.fingerprint, secret=True,
                                  passphrase=PASSPHRASE)
        print(privkey.replace('\r', ''), file=f)


def encrypt_file(sender, file_name, recipient, suffix='.pgp-enc'):
    gpg = gnupg.GPG(gnupghome=f'{sender}/.gnupg')
    with open(file_name, 'rb') as f:
        plaintext = f.read()

    ciphertext = gpg.encrypt(plaintext, recipients=[recipient])

    with open(file_name + suffix, 'w') as f:
        print(str(ciphertext).replace('\r', ''), file=f)


def decrypt_file(file_name, recipient):
    gpg = gnupg.GPG(gnupghome=f'{recipient}/.gnupg')
    with open(file_name, 'rb') as f:
        ciphertext = f.read()

    tmp = gpg.decrypt(ciphertext, passphrase=PASSPHRASE)
    return tmp.data


def setup_gpg(user):
    os.makedirs(f'{user}/.gnupg', exist_ok=True)
    gpg = gnupg.GPG(gnupghome=f'{user}/.gnupg')
    generate_keys(gpg, user)


def import_key(user, pubkey):
    gpg = gnupg.GPG(gnupghome=f'{user}/.gnupg')
    tmp = gpg.import_keys(pubkey)
    gpg.trust_keys(tmp.fingerprints, 'TRUST_ULTIMATE')


if __name__ == '__main__':
    import os

    plaintext = open('test.txt', 'rb').read()

    # generate public/private keys for andy and bob
    setup_gpg('andy')
    setup_gpg('bob')

    # import each other's public keys
    andy_pubkey = open('andy/.gnupg/gpg-public.asc').read()
    bob_pubkey = open('bob/.gnupg/gpg-public.asc').read()

    # keys must be imported to be able to encrypt messages to the person
    import_key('andy', bob_pubkey)
    import_key('bob', andy_pubkey)

    # the email is from setup_gpg -> generate_keys -> name_email
    encrypt_file('bob', 'test.txt', 'andy@example.com', suffix='.bob2andy')
    pt = decrypt_file('test.txt.bob2andy', 'andy')
    # print("PT:", repr(pt), repr(plaintext))
    assert plaintext == pt



# code from https://www.youtube.com/watch?v=bd5nsMscPo0&t=4s and copilot

import rsa
from cryptography.fernet import Fernet


def generate_keys():
    key = Fernet.generate_key()

    with open('symetric.key', 'wb') as f:
        f.write(key)

    public_key, private_key = rsa.newkeys(2048)
    with open('public.key', 'wb') as f:
        f.write(public_key.save_pkcs1('PEM'))

    with open('private.key', 'wb') as f:
        f.write(private_key.save_pkcs1('PEM'))


def encrypt_file(file_name, suffix='.enc'):
    with open('public.key', 'rb') as f:
        public_key = rsa.PublicKey.load_pkcs1(f.read())

    with open(file_name, 'rb') as f:
        data = f.read()

    encrypted_data = rsa.encrypt(data, public_key)

    with open(file_name + suffix, 'wb') as f:
        f.write(encrypted_data)


def decrypt_file(file_name):
    with open('private.key', 'rb') as f:
        private_key = rsa.PrivateKey.load_pkcs1(f.read())

    with open(file_name, 'rb') as f:
        data = f.read()

    decrypted_data = rsa.decrypt(data, private_key)

    with open(file_name + '.dec', 'wb') as f:
        f.write(decrypted_data)

    return decrypted_data


def encrypt_file_symetric(file_name, suffix='.enc'):
    with open('symetric.key', 'rb') as f:
        key = f.read()

    cipher = Fernet(key)

    with open(file_name, 'rb') as f:
        data = f.read()

    encrypted_data = cipher.encrypt(data)

    with open(file_name + suffix, 'wb') as f:
        f.write(encrypted_data)


def decrypt_file_symetric(file_name):
    with open('symetric.key', 'rb') as f:
        key = f.read()

    cipher = Fernet(key)

    with open(file_name, 'rb') as f:
        data = f.read()

    decrypted_data = cipher.decrypt(data)

    with open(file_name + '.dec', 'wb') as f:
        f.write(decrypted_data)

    return decrypted_data


if __name__ == '__main__':
    plaintext = open('test.txt', 'rb').read()
    # generate_keys()
    encrypt_file('test.txt', '.asymetric')
    assert plaintext == decrypt_file('test.txt.asymetric')
    encrypt_file_symetric('test.txt', '.symetric')
    assert plaintext == decrypt_file_symetric('test.txt.symetric')



import click

from . import seeqret_init


@click.group()
def cli():
    pass


@cli.command()
@click.option('--dir', default='.', help='Directory to initialize seeqret in')
def init():
    print('Initializing a new key pair')
    seeqret_init.secrets_init()


@cli.command()
def generate():
    print('Generating a new key pair')


@cli.command()
def encrypt():
    print('Encrypting a message')


@cli.command()
def decrypt():
    print('Decrypting a message')
